@@ -43,34 +43,31 @@
 
 #include <pthread.h>
 #include <unistd.h>
-#include <sys/types.h>
-#include <assert.h>
 #include <sys/stat.h>
-#include <stdint.h>
+#include <sys/types.h>
 
-#include <string>
-#include <map>
-#include <fstream>
-#include <cstdint>
-#include <iostream>
-#include <sstream>
-#include <vector>
-#include <memory>
 #include <algorithm>
-#include <iterator>
+#include <cassert>
+#include <cstdint>
 #include <cstring>
+#include <fstream>
+#include <iostream>
+#include <iterator>
+#include <map>
+#include <memory>
+#include <sstream>
+#include <string>
+#include <type_traits>
+#include <vector>
 
 #include "rocm_smi/rocm_smi_main.h"
 #include "rocm_smi/rocm_smi_device.h"
 #include "rocm_smi/rocm_smi.h"
 #include "rocm_smi/rocm_smi_exception.h"
 #include "rocm_smi/rocm_smi_utils.h"
-#include "rocm_smi/rocm_smi_kfd.h"
 #include "rocm_smi/rocm_smi_logger.h"
 #include "shared_mutex.h"  // NOLINT
 
-using namespace ROCmLogging;
-
 namespace amd {
 namespace smi {
 
@@ -85,6 +82,8 @@ static const char *kDevPerfLevelFName = "power_dpm_force_performance_level";
 static const char *kDevDevProdNameFName = "product_name";
 static const char *kDevDevProdNumFName = "product_number";
 static const char *kDevDevIDFName = "device";
+static const char* kDevXGMIPhysicalIDFName = "xgmi_physical_id";
+static const char *kDevDevRevIDFName = "revision";
 static const char *kDevVendorIDFName = "vendor";
 static const char *kDevSubSysDevIDFName = "subsystem_device";
 static const char *kDevSubSysVendorIDFName = "subsystem_vendor";
@@ -129,6 +128,7 @@ static const char *kDevAvailableComputePartitionFName =
                   "available_compute_partition";
 static const char *kDevComputePartitionFName = "current_compute_partition";
 static const char *kDevMemoryPartitionFName = "current_memory_partition";
+static const char *kDevAvailableMemoryPartitionFName = "available_memory_partition";
 
 // Firmware version files
 static const char *kDevFwVersionAsdFName = "fw_version/asd_fw_version";
@@ -138,6 +138,8 @@ static const char *kDevFwVersionMcFName = "fw_version/mc_fw_version";
 static const char *kDevFwVersionMeFName = "fw_version/me_fw_version";
 static const char *kDevFwVersionMecFName = "fw_version/mec_fw_version";
 static const char *kDevFwVersionMec2FName = "fw_version/mec2_fw_version";
+static const char *kDevFwVersionMesFName = "fw_version/mes_fw_version";
+static const char *kDevFwVersionMesKiqFName = "fw_version/mes_kiq_fw_version";
 static const char *kDevFwVersionPfpFName = "fw_version/pfp_fw_version";
 static const char *kDevFwVersionRlcFName = "fw_version/rlc_fw_version";
 static const char *kDevFwVersionRlcSrlcFName = "fw_version/rlc_srlc_fw_version";
@@ -238,6 +240,8 @@ static const std::map<DevInfoTypes, const char *> kDevAttribNameMap = {
     {kDevDevProdName, kDevDevProdNameFName},
     {kDevDevProdNum, kDevDevProdNumFName},
     {kDevDevID, kDevDevIDFName},
+    {kDevXGMIPhysicalID, kDevXGMIPhysicalIDFName},
+    {kDevDevRevID, kDevDevRevIDFName},
     {kDevVendorID, kDevVendorIDFName},
     {kDevSubSysDevID, kDevSubSysDevIDFName},
     {kDevSubSysVendorID, kDevSubSysVendorIDFName},
@@ -279,6 +283,8 @@ static const std::map<DevInfoTypes, const char *> kDevAttribNameMap = {
     {kDevFwVersionMe, kDevFwVersionMeFName},
     {kDevFwVersionMec, kDevFwVersionMecFName},
     {kDevFwVersionMec2, kDevFwVersionMec2FName},
+    {kDevFwVersionMes, kDevFwVersionMesFName},
+    {kDevFwVersionMesKiq, kDevFwVersionMesKiqFName},
     {kDevFwVersionPfp, kDevFwVersionPfpFName},
     {kDevFwVersionRlc, kDevFwVersionRlcFName},
     {kDevFwVersionRlcSrlc, kDevFwVersionRlcSrlcFName},
@@ -301,6 +307,7 @@ static const std::map<DevInfoTypes, const char *> kDevAttribNameMap = {
     {kDevAvailableComputePartition, kDevAvailableComputePartitionFName},
     {kDevComputePartition, kDevComputePartitionFName},
     {kDevMemoryPartition, kDevMemoryPartitionFName},
+    {kDevAvailableMemoryPartition, kDevAvailableMemoryPartitionFName},
 };
 
 static const std::map<rsmi_dev_perf_level, const char *> kDevPerfLvlMap = {
@@ -317,7 +324,7 @@ static const std::map<rsmi_dev_perf_level, const char *> kDevPerfLvlMap = {
     {RSMI_DEV_PERF_LEVEL_UNKNOWN, kDevPerfLevelUnknownStr},
 };
 
-static  std::map<DevInfoTypes, uint8_t> kDevInfoVarTypeToRSMIVariant = {
+static const std::map<DevInfoTypes, uint8_t> kDevInfoVarTypeToRSMIVariant = {
     // rsmi_memory_type_t
     {kDevMemTotGTT, RSMI_MEM_TYPE_GTT},
     {kDevMemTotVisVRAM, RSMI_MEM_TYPE_VIS_VRAM},
@@ -342,6 +349,8 @@ static  std::map<DevInfoTypes, uint8_t> kDevInfoVarTypeToRSMIVariant = {
     {kDevFwVersionMe, RSMI_FW_BLOCK_ME},
     {kDevFwVersionMec, RSMI_FW_BLOCK_MEC},
     {kDevFwVersionMec2, RSMI_FW_BLOCK_MEC2},
+    {kDevFwVersionMes, RSMI_FW_BLOCK_MES},
+    {kDevFwVersionMesKiq, RSMI_FW_BLOCK_MES_KIQ},
     {kDevFwVersionPfp, RSMI_FW_BLOCK_PFP},
     {kDevFwVersionRlc, RSMI_FW_BLOCK_RLC},
     {kDevFwVersionRlcSrlc, RSMI_FW_BLOCK_RLC_SRLC},
@@ -370,16 +379,96 @@ static  std::map<DevInfoTypes, uint8_t> kDevInfoVarTypeToRSMIVariant = {
     {kDevDFCountersAvailable, RSMI_EVNT_GRP_XGMI}
 };
 
+const std::map<DevInfoTypes, const char*>
+Device::devInfoTypesStrings = {
+  {kDevPerfLevel, "kDevPerfLevel"},
+  {kDevOverDriveLevel, "kDevOverDriveLevel"},
+  {kDevMemOverDriveLevel, "kDevMemOverDriveLevel"},
+  {kDevDevID, "kDevDevID"},
+  {kDevXGMIPhysicalID, "kDevXGMIPhysicalID"},
+  {kDevDevRevID, "kDevDevRevID"},
+  {kDevDevProdName, "kDevDevProdName"},
+  {kDevDevProdNum, "kDevDevProdNum"},
+  {kDevVendorID, "kDevVendorID"},
+  {kDevSubSysDevID, "kDevSubSysDevID"},
+  {kDevSubSysVendorID, "kDevSubSysVendorID"},
+  {kDevGPUMClk, "kDevGPUMClk"},
+  {kDevGPUSClk, "kDevGPUSClk"},
+  {kDevDCEFClk, "kDevDCEFClk"},
+  {kDevFClk, "kDevFClk"},
+  {kDevSOCClk, "kDevSOCClk"},
+  {kDevPCIEClk, "kDevPCIEClk"},
+  {kDevPowerProfileMode, "kDevPowerProfileMode"},
+  {kDevUsage, "kDevUsage"},
+  {kDevPowerODVoltage, "kDevPowerODVoltage"},
+  {kDevVBiosVer, "kDevVBiosVer"},
+  {kDevPCIEThruPut, "kDevPCIEThruPut"},
+  {kDevErrCntSDMA, "kDevErrCntSDMA"},
+  {kDevErrCntUMC, "kDevErrCntUMC"},
+  {kDevErrCntGFX, "kDevErrCntGFX"},
+  {kDevErrCntMMHUB, "kDevErrCntMMHUB"},
+  {kDevErrCntPCIEBIF, "kDevErrCntPCIEBIF"},
+  {kDevErrCntHDP, "kDevErrCntHDP"},
+  {kDevErrCntXGMIWAFL, "kDevErrCntXGMIWAFL"},
+  {kDevErrCntFeatures, "kDevErrCntFeatures"},
+  {kDevMemTotGTT, "kDevMemTotGTT"},
+  {kDevMemTotVisVRAM, "kDevMemTotVisVRAM"},
+  {kDevMemTotVRAM, "kDevMemTotVRAM"},
+  {kDevMemUsedGTT, "kDevMemUsedGTT"},
+  {kDevMemUsedVisVRAM, "kDevMemUsedVisVRAM"},
+  {kDevMemUsedVRAM, "kDevMemUsedVRAM"},
+  {kDevVramVendor, "kDevVramVendor"},
+  {kDevPCIEReplayCount, "kDevPCIEReplayCount"},
+  {kDevUniqueId, "kDevUniqueId"},
+  {kDevDFCountersAvailable, "kDevDFCountersAvailable"},
+  {kDevMemBusyPercent, "kDevMemBusyPercent"},
+  {kDevXGMIError, "kDevXGMIError"},
+  {kDevFwVersionAsd, "kDevFwVersionAsd"},
+  {kDevFwVersionCe, "kDevFwVersionCe"},
+  {kDevFwVersionDmcu, "kDevFwVersionDmcu"},
+  {kDevFwVersionMc, "kDevFwVersionMc"},
+  {kDevFwVersionMe, "kDevFwVersionMe"},
+  {kDevFwVersionMec, "kDevFwVersionMec"},
+  {kDevFwVersionMec2, "kDevFwVersionMec2"},
+  {kDevFwVersionMes, "kDevFwVersionMes"},
+  {kDevFwVersionMesKiq, "kDevFwVersionMesKiq"},
+  {kDevFwVersionPfp, "kDevFwVersionPfp"},
+  {kDevFwVersionRlc, "kDevFwVersionRlc"},
+  {kDevFwVersionRlcSrlc, "kDevFwVersionRlcSrlc"},
+  {kDevFwVersionRlcSrlg, "kDevFwVersionRlcSrlg"},
+  {kDevFwVersionRlcSrls, "kDevFwVersionRlcSrls"},
+  {kDevFwVersionSdma, "kDevFwVersionSdma"},
+  {kDevFwVersionSdma2, "kDevFwVersionSdma2"},
+  {kDevFwVersionSmc, "kDevFwVersionSmc"},
+  {kDevFwVersionSos, "kDevFwVersionSos"},
+  {kDevFwVersionTaRas, "kDevFwVersionTaRas"},
+  {kDevFwVersionTaXgmi, "kDevFwVersionTaXgmi"},
+  {kDevFwVersionUvd, "kDevFwVersionUvd"},
+  {kDevFwVersionVce, "kDevFwVersionVce"},
+  {kDevFwVersionVcn, "kDevFwVersionVcn"},
+  {kDevSerialNumber, "kDevSerialNumber"},
+  {kDevMemPageBad, "kDevMemPageBad"},
+  {kDevNumaNode, "kDevNumaNode"},
+  {kDevGpuMetrics, "kDevGpuMetrics"},
+  {kDevGpuReset, "kDevGpuReset"},
+  {kDevAvailableComputePartition, "kDevAvailableComputePartition"},
+  {kDevComputePartition, "kDevComputePartition"},
+  {kDevMemoryPartition, "kDevMemoryPartition"},
+  {kDevAvailableMemoryPartition, "kDevAvailableMemoryPartition"},
+};
+
 static const std::map<const char *, dev_depends_t> kDevFuncDependsMap = {
     // Functions with only mandatory dependencies
   {"rsmi_dev_vram_vendor_get",           {{kDevVramVendorFName}, {}}},
   {"rsmi_dev_id_get",                    {{kDevDevIDFName}, {}}},
+  {"rsmi_dev_xgmi_physical_id_get",      {{kDevXGMIPhysicalIDFName}, {}}},
+  {"rsmi_dev_revision_get",              {{kDevDevRevIDFName}, {}}},
   {"rsmi_dev_vendor_id_get",             {{kDevVendorIDFName}, {}}},
-
   {"rsmi_dev_name_get",                  {{kDevVendorIDFName,
                                            kDevDevIDFName}, {}}},
   {"rsmi_dev_sku_get",                   {{kDevDevProdNumFName}, {}}},
-  {"rsmi_dev_brand_get",                 {{kDevVendorIDFName}, {}}},
+  {"rsmi_dev_brand_get",                 {{kDevVendorIDFName,
+                                           kDevVBiosVerFName}, {}}},
   {"rsmi_dev_vendor_name_get",           {{kDevVendorIDFName}, {}}},
   {"rsmi_dev_serial_number_get",         {{kDevSerialNumberFName}, {}}},
   {"rsmi_dev_subsystem_id_get",          {{kDevSubSysDevIDFName}, {}}},
@@ -476,6 +565,8 @@ static const std::map<const char *, dev_depends_t> kDevFuncDependsMap = {
                                         kDevFwVersionMe,
                                         kDevFwVersionMec,
                                         kDevFwVersionMec2,
+                                        kDevFwVersionMes,
+                                        kDevFwVersionMesKiq,
                                         kDevFwVersionPfp,
                                         kDevFwVersionRlc,
                                         kDevFwVersionRlcSrlc,
@@ -519,7 +610,7 @@ static const std::map<const char *, dev_depends_t> kDevFuncDependsMap = {
 
 Device::Device(std::string p, RocmSMI_env_vars const *e) :
             monitor_(nullptr), path_(p), env_(e), evt_notif_anon_fd_(-1),
-                                                   gpu_metrics_ver_{0, 0, 0} {
+                                                   m_gpu_metrics_header{0, 0, 0} {
 #ifndef DEBUG
     env_ = nullptr;
 #endif
@@ -589,17 +680,19 @@ int Device::openSysfsFileStream(DevInfoTypes type, T *fs, const char *str) {
   bool reg_file;
 
   int ret = isRegularFile(sysfs_path, &reg_file);
-
   if (ret != 0) {
-    ss << "File did not exist - SYSFS file (" << sysfs_path
-       << ") for DevInfoInfoType (" << RocmSMI::devInfoTypesStrings.at(type)
+    ss << __PRETTY_FUNCTION__ << " | Issue: File did not exist - SYSFS file ("
+       << sysfs_path
+       << ") for DevInfoInfoType (" << get_type_string(type)
        << "), returning " << std::to_string(ret);
     LOG_ERROR(ss);
     return ret;
   }
   if (!reg_file) {
-    ss << "File is not a regular file - SYSFS file (" << sysfs_path << ") for "
-       << "DevInfoInfoType (" << RocmSMI::devInfoTypesStrings.at(type) << "),"
+    ss << __PRETTY_FUNCTION__
+       << " | Issue: File is not a regular file - SYSFS file ("
+       << sysfs_path << ") for "
+       << "DevInfoInfoType (" << get_type_string(type) << "),"
        << " returning ENOENT (" << std::strerror(ENOENT) << ")";
     LOG_ERROR(ss);
     return ENOENT;
@@ -608,16 +701,18 @@ int Device::openSysfsFileStream(DevInfoTypes type, T *fs, const char *str) {
   fs->open(sysfs_path);
 
   if (!fs->is_open()) {
-    ss << "Could not open - SYSFS file (" << sysfs_path << ") for "
-       << "DevInfoInfoType (" << RocmSMI::devInfoTypesStrings.at(type) << "), "
+    ss << __PRETTY_FUNCTION__
+       << " | Issue: Could not open - SYSFS file (" << sysfs_path << ") for "
+       << "DevInfoInfoType (" << get_type_string(type) << "), "
        << ", returning " << std::to_string(errno) << " ("
        << std::strerror(errno) << ")";
     LOG_ERROR(ss);
     return errno;
   }
 
-  ss << "Successfully opened SYSFS file (" << sysfs_path
-     << ") for DevInfoInfoType (" << RocmSMI::devInfoTypesStrings.at(type)
+  ss << __PRETTY_FUNCTION__ << " | Successfully opened SYSFS file ("
+     << sysfs_path
+     << ") for DevInfoInfoType (" << get_type_string(type)
      << ")";
   LOG_INFO(ss);
   return 0;
@@ -634,7 +729,7 @@ int Device::readDebugInfoStr(DevInfoTypes type, std::string *retStr) {
   ret = openDebugFileStream(type, &fs);
   if (ret != 0) {
     ss << "Could not read debugInfoStr for DevInfoType ("
-     << RocmSMI::devInfoTypesStrings.at(type)<< "), returning "
+     << get_type_string(type) << "), returning "
      << std::to_string(ret);
     LOG_ERROR(ss);
     return ret;
@@ -648,7 +743,7 @@ int Device::readDebugInfoStr(DevInfoTypes type, std::string *retStr) {
   fs.close();
 
   ss << "Successfully read debugInfoStr for DevInfoType ("
-     << RocmSMI::devInfoTypesStrings.at(type)<< "), retString= " << *retStr;
+     << get_type_string(type) << "), retString= " << *retStr;
   LOG_INFO(ss);
 
   return 0;
@@ -664,33 +759,52 @@ int Device::readDevInfoStr(DevInfoTypes type, std::string *retStr) {
   ret = openSysfsFileStream(type, &fs);
   if (ret != 0) {
     ss << "Could not read device info string for DevInfoType ("
-     << RocmSMI::devInfoTypesStrings.at(type)<< "), returning "
+     << get_type_string(type) << "), returning "
      << std::to_string(ret);
     LOG_ERROR(ss);
     return ret;
   }
 
   fs >> *retStr;
-  std::string info = "Successfully read device info string for DevInfoType (" +
-                      RocmSMI::devInfoTypesStrings.at(type) + "): " +
-                      *retStr;
-  LOG_INFO(info);
   fs.close();
-
+  ss << __PRETTY_FUNCTION__
+     << "Successfully read device info string for DevInfoType ("
+     << get_type_string(type) << "): " + *retStr
+     << " | "
+     << (fs.is_open() ? " File stream is opened" : " File stream is closed")
+     << " | " << (fs.bad() ? "[ERROR] Bad read operation" :
+     "[GOOD] No bad bit read, successful read operation")
+     << " | " << (fs.fail() ? "[ERROR] Failed read - format error" :
+     "[GOOD] No fail - Successful read operation")
+     << " | " << (fs.eof() ? "[ERROR] Failed read - EOF error" :
+     "[GOOD] No eof - Successful read operation")
+     << " | " << (fs.good() ? "[GOOD] read good - Successful read operation" :
+     "[ERROR] Failed read - good error");
+  LOG_INFO(ss);
   return 0;
 }
 
-int Device::writeDevInfoStr(DevInfoTypes type, std::string valStr) {
-  auto tempPath = path_;
+int Device::writeDevInfoStr(DevInfoTypes type, std::string valStr,
+                           bool returnWriteErr) {
+  // returnWriteErr = false, backwards compatability (old calls)
+  // returnWriteErr = true, improvement - allows us to detect errors
+  // when writing to file
+  // (such as EBUSY)
+  auto sysfs_path = path_;
+  sysfs_path += "/device/";
+  sysfs_path += kDevAttribNameMap.at(type);
   std::ofstream fs;
   int ret;
   std::ostringstream ss;
 
-  fs.rdbuf()->pubsetbuf(0,0);
+  fs.flush();
+  fs.rdbuf()->pubsetbuf(0, 0);
   ret = openSysfsFileStream(type, &fs, valStr.c_str());
   if (ret != 0) {
-    ss << "Could not write device info string (" << valStr
-       << ") for DevInfoType (" << RocmSMI::devInfoTypesStrings.at(type)
+    fs.close();
+    ss << __PRETTY_FUNCTION__ << " | Issue: Could not open fileStream; "
+       << "Could not write device info string (" << valStr
+       << ") for DevInfoType (" << get_type_string(type)
        << "), returning " << std::to_string(ret);
     LOG_ERROR(ss);
     return ret;
@@ -698,19 +812,39 @@ int Device::writeDevInfoStr(DevInfoTypes type, std::string valStr) {
 
   // We'll catch any exceptions in rocm_smi.cc code.
   if (fs << valStr) {
+    fs.flush();
+    fs.close();
     ss << "Successfully wrote device info string (" << valStr
-       << ") for DevInfoType (" << RocmSMI::devInfoTypesStrings.at(type)
+       << ") for DevInfoType (" << get_type_string(type)
        << "), returning RSMI_STATUS_SUCCESS";
     LOG_INFO(ss);
     ret = RSMI_STATUS_SUCCESS;
   } else {
-    ss << "Could not write device info string (" << valStr
-       << ") for DevInfoType (" << RocmSMI::devInfoTypesStrings.at(type)
-       << "), returning RSMI_STATUS_NOT_SUPPORTED";
+    if (returnWriteErr) {
+      ret = errno;
+    } else {
+      ret = RSMI_STATUS_NOT_SUPPORTED;
+    }
+    fs.flush();
+    fs.close();
+    ss << __PRETTY_FUNCTION__ << " | Issue: Could not write to file; "
+       << "Could not write device info string (" << valStr
+       << ") for DevInfoType (" << get_type_string(type)
+       << "), returning " << getRSMIStatusString(ErrnoToRsmiStatus(ret));
+    ss << " | "
+       << (fs.is_open() ? "[ERROR] File stream open" :
+          "[GOOD] File stream closed")
+       << " | " << (fs.bad() ? "[ERROR] Bad write operation" :
+                    "[GOOD] No bad bit write, successful write operation")
+       << " | " << (fs.fail() ? "[ERROR] Failed write - format error" :
+                    "[GOOD] No fail - Successful write operation")
+       << " | " << (fs.eof() ? "[ERROR] Failed write - EOF error" :
+                    "[GOOD] No eof - Successful write operation")
+       << " | " << (fs.good() ?
+                   "[GOOD] Write good - Successful write operation" :
+                   "[ERROR] Failed write - good error");
     LOG_ERROR(ss);
-    ret = RSMI_STATUS_NOT_SUPPORTED;
   }
-  fs.close();
 
   return ret;
 }
@@ -749,6 +883,9 @@ int Device::writeDevInfo(DevInfoTypes type, uint64_t val) {
 }
 
 int Device::writeDevInfo(DevInfoTypes type, std::string val) {
+  auto sysfs_path = path_;
+  sysfs_path += "/device/";
+  sysfs_path += kDevAttribNameMap.at(type);
   switch (type) {
     case kDevGPUMClk:
     case kDevDCEFClk:
@@ -757,9 +894,10 @@ int Device::writeDevInfo(DevInfoTypes type, std::string val) {
     case kDevPCIEClk:
     case kDevPowerODVoltage:
     case kDevSOCClk:
+      return writeDevInfoStr(type, val);
     case kDevComputePartition:
     case kDevMemoryPartition:
-      return writeDevInfoStr(type, val);
+      return writeDevInfoStr(type, val, true);
 
     default:
       return EINVAL;
@@ -778,20 +916,29 @@ int Device::readDevInfoLine(DevInfoTypes type, std::string *line) {
   ret = openSysfsFileStream(type, &fs);
   if (ret != 0) {
     ss << "Could not read DevInfoLine for DevInfoType ("
-       << RocmSMI::devInfoTypesStrings.at(type) << ")";
+       << get_type_string(type) << ")";
     LOG_ERROR(ss);
     return ret;
   }
 
   std::getline(fs, *line);
   ss << "Successfully read DevInfoLine for DevInfoType ("
-     << RocmSMI::devInfoTypesStrings.at(type) << "), returning *line = "
+     << get_type_string(type) << "), returning *line = "
      << *line;
   LOG_INFO(ss);
 
   return 0;
 }
 
+const char* Device::get_type_string(DevInfoTypes type) {
+  auto ite = devInfoTypesStrings.find(type);
+  if (ite != devInfoTypesStrings.end()) {
+    return ite->second;
+  }
+
+  return "Unknown";
+}
+
 int Device::readDevInfoBinary(DevInfoTypes type, std::size_t b_size,
                                 void *p_binary_data) {
   auto sysfs_path = path_;
@@ -800,10 +947,11 @@ int Device::readDevInfoBinary(DevInfoTypes type, std::size_t b_size,
   FILE *ptr;
   sysfs_path += "/device/";
   sysfs_path += kDevAttribNameMap.at(type);
-  ptr  = fopen(sysfs_path.c_str(), "rb");
+
+  ptr = fopen(sysfs_path.c_str(), "rb");
   if (!ptr) {
     ss << "Could not read DevInfoBinary for DevInfoType ("
-       << RocmSMI::devInfoTypesStrings.at(type) << ")"
+       << get_type_string(type) << ")"
        << " - SYSFS (" << sysfs_path << ")"
        << ", returning " << std::to_string(errno) << " ("
        << std::strerror(errno) << ")";
@@ -815,16 +963,30 @@ int Device::readDevInfoBinary(DevInfoTypes type, std::size_t b_size,
   fclose(ptr);
   if ((num*b_size) != b_size) {
     ss << "Could not read DevInfoBinary for DevInfoType ("
-       << RocmSMI::devInfoTypesStrings.at(type) << ") - SYSFS ("
-       << sysfs_path << "), binary size error, "
+       << get_type_string(type) << ") - SYSFS ("
+       << sysfs_path << "), binary size error; "
+       << "[buff: "
+       << p_binary_data
+       << " size: "
+       << b_size
+       << " read: "
+       << num
+       << "]"
        << ", returning ENOENT (" << std::strerror(ENOENT) << ")";
     LOG_ERROR(ss);
     return ENOENT;
   }
-  ss << "Successfully read DevInfoBinary for DevInfoType ("
-     << RocmSMI::devInfoTypesStrings.at(type) << ") - SYSFS ("
-     << sysfs_path << "), returning binaryData = " << p_binary_data;
-  LOG_INFO(ss);
+  if (ROCmLogging::Logger::getInstance()->isLoggerEnabled()) {
+    ss << "Successfully read DevInfoBinary for DevInfoType ("
+       << get_type_string(type) << ") - SYSFS ("
+       << sysfs_path << "), returning binaryData = " << p_binary_data
+       << "; byte_size = " << std::dec << static_cast<int>(b_size);
+
+    std::string metricDescription = "AMD SMI GPU METRICS (16-byte width), "
+                                  + sysfs_path;
+    logHexDump(metricDescription.c_str(), p_binary_data, b_size, 16);
+    LOG_INFO(ss);
+  }
   return 0;
 }
 
@@ -847,34 +1009,35 @@ int Device::readDevInfoMultiLineStr(DevInfoTypes type,
     retVec->push_back(line);
   }
 
-  if (retVec->size() == 0) {
+  if (retVec->empty()) {
     ss << "Read devInfoMultiLineStr for DevInfoType ("
-       << RocmSMI::devInfoTypesStrings.at(type) << ")"
+       << get_type_string(type) << ")"
        << ", but contained no string lines";
-    LOG_INFO(ss);
-    return 0;
+    LOG_ERROR(ss);
+    return ENXIO;
   }
   // Remove any *trailing* empty (whitespace) lines
-  while (retVec->size() != 0 &&
+  while (!retVec->empty() &&
         retVec->back().find_first_not_of(" \t\n\v\f\r") == std::string::npos) {
     retVec->pop_back();
   }
 
   // allow logging output of multiline strings
-  for (auto l: *retVec) {
+  for (const auto& l: *retVec) {
     allLines += "\n" + l;
   }
 
   if (!allLines.empty()) {
     ss << "Successfully read devInfoMultiLineStr for DevInfoType ("
-       << RocmSMI::devInfoTypesStrings.at(type) << ") "
+       << get_type_string(type) << ") "
        << ", returning lines read = " << allLines;
     LOG_INFO(ss);
   } else {
     ss << "Read devInfoMultiLineStr for DevInfoType ("
-       << RocmSMI::devInfoTypesStrings.at(type) << ")"
+       << get_type_string(type) << ")"
        << ", but lines were empty";
     LOG_INFO(ss);
+    return ENXIO;
   }
   return 0;
 }
@@ -885,20 +1048,23 @@ int Device::readDevInfo(DevInfoTypes type, uint64_t *val) {
   std::string tempStr;
   int ret;
   int tmp_val;
+  std::ostringstream ss;
 
   switch (type) {
     case kDevDevID:
+    case kDevDevRevID:
     case kDevSubSysDevID:
     case kDevSubSysVendorID:
     case kDevVendorID:
     case kDevErrCntFeatures:
+    case kDevXGMIPhysicalID:
       ret = readDevInfoStr(type, &tempStr);
       RET_IF_NONZERO(ret);
 
-      if (tempStr == "") {
+      if (tempStr.empty()) {
         return EINVAL;
       }
-      tmp_val = std::stoi(tempStr, 0, 16);
+      tmp_val = std::stoi(tempStr, nullptr, 16);
       if (tmp_val < 0) {
         return EINVAL;
       }
@@ -918,13 +1084,12 @@ int Device::readDevInfo(DevInfoTypes type, uint64_t *val) {
     case kDevDFCountersAvailable:
     case kDevMemBusyPercent:
     case kDevXGMIError:
-    case kDevNumaNode:
       ret = readDevInfoStr(type, &tempStr);
       RET_IF_NONZERO(ret);
-      if (tempStr == "") {
+      if (tempStr.empty()) {
         return EINVAL;
       }
-      *val = std::stoul(tempStr, 0);
+      *val = std::stoul(tempStr, nullptr);
       break;
 
     case kDevUniqueId:
@@ -935,6 +1100,8 @@ int Device::readDevInfo(DevInfoTypes type, uint64_t *val) {
     case kDevFwVersionMe:
     case kDevFwVersionMec:
     case kDevFwVersionMec2:
+    case kDevFwVersionMes:
+    case kDevFwVersionMesKiq:
     case kDevFwVersionPfp:
     case kDevFwVersionRlc:
     case kDevFwVersionRlcSrlc:
@@ -951,10 +1118,10 @@ int Device::readDevInfo(DevInfoTypes type, uint64_t *val) {
     case kDevFwVersionVcn:
       ret = readDevInfoStr(type, &tempStr);
       RET_IF_NONZERO(ret);
-      if (tempStr == "") {
+      if (tempStr.empty()) {
         return EINVAL;
       }
-      *val = std::stoul(tempStr, 0, 16);
+      *val = std::stoul(tempStr, nullptr, 16);
       break;
 
     case kDevGpuReset:
@@ -1025,6 +1192,7 @@ int Device::readDevInfo(DevInfoTypes type, std::string *val) {
     case kDevDevProdName:
     case kDevDevProdNum:
     case kDevDevID:
+    case kDevDevRevID:
     case kDevSubSysDevID:
     case kDevSubSysVendorID:
     case kDevVendorID:
@@ -1035,6 +1203,9 @@ int Device::readDevInfo(DevInfoTypes type, std::string *val) {
     case kDevAvailableComputePartition:
     case kDevComputePartition:
     case kDevMemoryPartition:
+    case kDevNumaNode:
+    case kDevXGMIPhysicalID:
+    case kDevAvailableMemoryPartition:
       return readDevInfoStr(type, val);
       break;
 
@@ -1089,7 +1260,7 @@ void Device::DumpSupportedFunctions(void) {
 }
 
 void Device::fillSupportedFuncs(void) {
-  if (supported_funcs_.size() != 0) {
+  if (!supported_funcs_.empty()) {
     return;
   }
   std::map<const char *, dev_depends_t>::const_iterator it =
@@ -1125,7 +1296,7 @@ void Device::fillSupportedFuncs(void) {
     std::vector<DevInfoTypes>::const_iterator var =
                                                   it->second.variants.begin();
 
-    if (it->second.variants.size() == 0) {
+    if (it->second.variants.empty()) {
       supported_funcs_[it->first] = nullptr;
       it++;
       continue;
@@ -1141,7 +1312,7 @@ void Device::fillSupportedFuncs(void) {
       (*supported_variants)[kDevInfoVarTypeToRSMIVariant.at(*var)] = nullptr;
     }
 
-    if ((*supported_variants).size() > 0) {
+    if (!(*supported_variants).empty()) {
       supported_funcs_[it->first] = supported_variants;
     }
 
@@ -1189,65 +1360,148 @@ bool Device::DeviceAPISupported(std::string name, uint64_t variant,
 
     if (sub_variant == RSMI_DEFAULT_VARIANT) {
       return true;
-    } else {  // sub_variant != RSMI_DEFAULT_VARIANT
-      // if variant is != RSMI_DEFAULT_VARIANT, we should not have a nullptr
-      assert(var_it->second != nullptr);
+    }
+    // sub_variant != RSMI_DEFAULT_VARIANT
+    // if variant is != RSMI_DEFAULT_VARIANT, we should not have a nullptr
+    assert(var_it->second != nullptr);
 
-      return subvariant_match(&(var_it->second), sub_variant);
-    }
-  } else {  // variant == RSMI_DEFAULT_VARIANT
-    if (func_it->second != nullptr) {
-      var_it = func_it->second->find(variant);
-    }
-    if (sub_variant == RSMI_DEFAULT_VARIANT) {
-      return true;
-    } else {  // sub_variant != RSMI_DEFAULT_VARIANT
-      if (func_it->second == nullptr) {
-        return false;
-      }
-      return subvariant_match(&(var_it->second), sub_variant);
-    }
+    return subvariant_match(&(var_it->second), sub_variant);
   }
-  assert(false);  // We should not reach here
-
-  return false;
+  // variant == RSMI_DEFAULT_VARIANT
+  if (func_it->second != nullptr) {
+    var_it = func_it->second->find(variant);
+  }
+  if (sub_variant == RSMI_DEFAULT_VARIANT) {
+    return true;
+  }
+  // sub_variant != RSMI_DEFAULT_VARIANT
+  if (func_it->second == nullptr) {
+    return false;
+  }
+  return subvariant_match(&(var_it->second), sub_variant);
 }
 
 rsmi_status_t Device::restartAMDGpuDriver(void) {
   REQUIRE_ROOT_ACCESS
+  std::ostringstream ss;
   bool restartSuccessful = true;
   bool success = false;
-  std::string out = "";
+  std::string out;
   bool wasGdmServiceActive = false;
+  bool restartInProgress = true;
+  bool isRestartInProgress = true;
+  bool isAMDGPUModuleLive = false;
+  bool restartGDM = false;
+  std::string captureRestartErr;
+  const int kTimeToWaitForDriverMSec = 1000;
 
   // sudo systemctl is-active gdm
   // we do not care about the success of checking if gdm is active
-  std::tie(success, out) = executeCommand("systemctl is-active gdm");
-  (out == "active") ? (restartSuccessful &= success) :
-                         (restartSuccessful = true);
+  std::tie(success, out) = executeCommand("systemctl is-active gdm", true);
+  (out == "active") ? (restartGDM = true) : (restartGDM = false);
+  ss << __PRETTY_FUNCTION__ << " | systemctl is-active gdm: out = "
+     << out << "; success = " << (success ? "True" : "False");
+  LOG_INFO(ss);
 
   // if gdm is active -> sudo systemctl stop gdm
-  // TODO: are are there other display manager's we need to take into account?
-  // see https://en.wikipedia.org/wiki/GNOME_Display_Manager
-  if (success && (out == "active")) {
+  // TODO(AMD_SMI_team): are are there other display manager's we need to take into account?
+  // see https://help.gnome.org/admin/gdm/stable/overview.html.en_GB
+  if (success && (out == "active") && (restartGDM)) {
     wasGdmServiceActive = true;
-    std::tie(success, out) = executeCommand("systemctl stop gdm&", false);
-    restartSuccessful &= success;
+    std::tie(success, out) = executeCommand("systemctl stop gdm&", true);
+    ss << __PRETTY_FUNCTION__ << " | systemctl stop gdm&: out = "
+    << out << "; success = " << (success ? "True" : "False");
+    LOG_INFO(ss);
+  } else {
+    success = true;  // ignore failures to restart gdm
   }
 
+  ss << __PRETTY_FUNCTION__ << " | B4 modprobing anything!!! out = "
+     << out << "; success = " << (success ? "True" : "False")
+     << "; restartSuccessful = " << (restartSuccessful ? "True" : "False")
+     << "; captureRestartErr = " << captureRestartErr;
+  LOG_INFO(ss);
+
   // sudo modprobe -r amdgpu
   // sudo modprobe amdgpu
-  std::tie(success, out) =
-    executeCommand("modprobe -r amdgpu && modprobe amdgpu&", false);
+  std::tie(success, out) = executeCommand(
+    "modprobe -r -v amdgpu >/dev/null 2>&1 && modprobe -v amdgpu >/dev/null 2>&1", true);
   restartSuccessful &= success;
+  captureRestartErr = out;
+  ss << __PRETTY_FUNCTION__ << " | modprobe -r -v amdgpu && modprobe -v amdgpu: out = "
+     << out << "; success = " << (success ? "True" : "False")
+     << "; restartSuccessful = " << (restartSuccessful ? "True" : "False")
+     << "; captureRestartErr = " << captureRestartErr;
+  LOG_INFO(ss);
 
   // if gdm was active -> sudo systemctl start gdm
-  if (wasGdmServiceActive) {
-    std::tie(success, out) = executeCommand("systemctl start gdm&", false);
-    restartSuccessful &= success;
+  // We don't care if successful or not, just try to restart as a courtesy
+  if (wasGdmServiceActive && restartGDM) {
+    std::tie(success, out) = executeCommand("systemctl start gdm&", true);
+    ss << __PRETTY_FUNCTION__ << " | systemctl start gdm&: out = "
+    << out << "; success = " << (success ? "True" : "False");
+    LOG_INFO(ss);
   }
 
-  return (restartSuccessful ? RSMI_STATUS_SUCCESS :
+  // Return early if there was an issue restarting amdgpu
+  if (!restartSuccessful) {
+    ss << __PRETTY_FUNCTION__ << " | [WARNING] Issue found during amdgpu restart: "
+    << captureRestartErr << "; retartSuccessful: " << (restartSuccessful ? "True" : "False");
+    LOG_INFO(ss);
+    return RSMI_STATUS_AMDGPU_RESTART_ERR;
+  }
+
+  // wait for amdgpu module to come back up
+  rsmi_status_t status = Device::isRestartInProgress(&isRestartInProgress,
+                                                    &isAMDGPUModuleLive);
+  int maxLoops = 10;  // wait a max of 10 sec
+  while (status != RSMI_STATUS_SUCCESS) {
+    maxLoops -= 1;
+    if (maxLoops == 0) {
+      break;
+    }
+    amd::smi::system_wait(kTimeToWaitForDriverMSec);
+    status = Device::isRestartInProgress(&isRestartInProgress,
+                                         &isAMDGPUModuleLive);
+  }
+
+  return ((restartSuccessful && (!isRestartInProgress && isAMDGPUModuleLive)) ?
+          RSMI_STATUS_SUCCESS :
+          RSMI_STATUS_AMDGPU_RESTART_ERR);
+}
+
+rsmi_status_t Device::isRestartInProgress(bool *isRestartInProgress,
+                                          bool *isAMDGPUModuleLive) {
+  REQUIRE_ROOT_ACCESS
+  std::ostringstream ss;
+  bool restartSuccessful = true;
+  bool success = false;
+  std::string out;
+  bool deviceRestartInProgress = true;    // Assume in progress, we intend to disprove
+  bool isSystemAMDGPUModuleLive = false;  // Assume AMD GPU module is not live,
+                                          //  we intend to disprove
+
+  // wait for amdgpu module to come back up
+  std::tie(success, out) = executeCommand("cat /sys/module/amdgpu/initstate", true);
+  ss << __PRETTY_FUNCTION__
+     << " | success = " << (success ? "True" : "False")
+     << " | out = " << out;
+  LOG_DEBUG(ss);
+  if ((success == true) && (!out.empty())) {
+    isSystemAMDGPUModuleLive = containsString(out, "live");
+  }
+  if (isAMDGPUModuleLive) {
+    deviceRestartInProgress = false;
+  }
+  *isRestartInProgress = deviceRestartInProgress;
+  *isAMDGPUModuleLive = isSystemAMDGPUModuleLive;
+  ss << __PRETTY_FUNCTION__
+     << " | *isRestartInProgress = " << (*isRestartInProgress ? "True":"False")
+     << " | *isAMDGPUModuleLive = " << (*isAMDGPUModuleLive ? "True":"False")
+     << " | out = " << out;
+  LOG_DEBUG(ss);
+
+  return ((*isAMDGPUModuleLive && !*isRestartInProgress) ? RSMI_STATUS_SUCCESS :
           RSMI_STATUS_AMDGPU_RESTART_ERR);
 }
 
@@ -1257,7 +1511,7 @@ template <typename T> rsmi_status_t storeParameter(uint32_t dv_ind);
 // Uses template specialization, to restrict types to identify
 // calls needed to complete the function.
 // typename - restricted to
-// rsmi_compute_partition_type_t or rsmi_compute_partition_type_t
+// rsmi_compute_partition_type_t or rsmi_memory_partition_type_t
 // dv_ind - device index
 // tempFileName - base file name
 template <>
@@ -1271,9 +1525,9 @@ rsmi_status_t storeParameter<rsmi_compute_partition_type_t>(uint32_t dv_ind) {
   if (doesFileExist) {
     return returnStatus;
   }
-  uint32_t length = 128;
-  char data[length];
-  rsmi_status_t ret = rsmi_dev_compute_partition_get(dv_ind, data, length);
+  const uint32_t kLen = 128;
+  char data[kLen];
+  rsmi_status_t ret = rsmi_dev_compute_partition_get(dv_ind, data, kLen);
   rsmi_status_t storeRet;
 
   if (ret == RSMI_STATUS_SUCCESS) {
@@ -1297,31 +1551,32 @@ rsmi_status_t storeParameter<rsmi_compute_partition_type_t>(uint32_t dv_ind) {
 // Uses template specialization, to restrict types to identify
 // calls needed to complete the function.
 // typename - restricted to
-// rsmi_compute_partition_type_t or rsmi_compute_partition_type_t
+// rsmi_compute_partition_type_t or rsmi_memory_partition_type_t
 // dv_ind - device index
 // tempFileName - base file name
-template <> rsmi_status_t storeParameter<rsmi_nps_mode_type_t>(uint32_t dv_ind) {
+template <>
+rsmi_status_t storeParameter<rsmi_memory_partition_type_t>(uint32_t dv_ind) {
   rsmi_status_t returnStatus = RSMI_STATUS_SUCCESS;
-  uint32_t length = 128;
-  char data[length];
+  uint32_t kDatalength = 128;
+  char data[kDatalength];
   bool doesFileExist;
   std::tie(doesFileExist, std::ignore) = readTmpFile(dv_ind, "boot",
-                                                     "nps_mode");
+                                                     "memory_partition");
   // if temporary file exists -> we do not need to store anything new
   // if not, read & store the state value
   if (doesFileExist) {
     return returnStatus;
   }
-  rsmi_status_t ret = rsmi_dev_nps_mode_get(dv_ind, data, length);
+  rsmi_status_t ret = rsmi_dev_memory_partition_get(dv_ind, data, kDatalength);
   rsmi_status_t storeRet;
 
   if (ret == RSMI_STATUS_SUCCESS) {
-    storeRet = storeTmpFile(dv_ind, "nps_mode", "boot", data);
+    storeRet = storeTmpFile(dv_ind, "memory_partition", "boot", data);
   } else if (ret == RSMI_STATUS_NOT_SUPPORTED) {
     // not supported is ok
-    storeRet = storeTmpFile(dv_ind, "nps_mode", "boot", "UNKNOWN");
+    storeRet = storeTmpFile(dv_ind, "memory_partition", "boot", "UNKNOWN");
   } else {
-    storeRet = storeTmpFile(dv_ind, "nps_mode", "boot", "UNKNOWN");
+    storeRet = storeTmpFile(dv_ind, "memory_partition", "boot", "UNKNOWN");
     returnStatus = ret;
   }
 
@@ -1335,9 +1590,9 @@ template <> rsmi_status_t storeParameter<rsmi_nps_mode_type_t>(uint32_t dv_ind)
 rsmi_status_t Device::storeDevicePartitions(uint32_t dv_ind) {
   rsmi_status_t returnStatus = RSMI_STATUS_SUCCESS;
   returnStatus = storeParameter<rsmi_compute_partition_type_t>(dv_ind);
-  rsmi_status_t npsRet = storeParameter<rsmi_nps_mode_type_t>(dv_ind);
-  if (returnStatus == RSMI_STATUS_SUCCESS) { // only record earliest error
-    returnStatus = npsRet;
+  rsmi_status_t ret = storeParameter<rsmi_memory_partition_type_t>(dv_ind);
+  if (returnStatus == RSMI_STATUS_SUCCESS) {  // only record earliest error
+    returnStatus = ret;
   }
   return returnStatus;
 }
@@ -1366,13 +1621,15 @@ std::string Device::readBootPartitionState<rsmi_compute_partition_type_t>(
 // or rsmi_compute_partition_type_t
 // dv_ind - device index
 template <>
-std::string Device::readBootPartitionState<rsmi_nps_mode_type_t>(
+std::string Device::readBootPartitionState<rsmi_memory_partition_type_t>(
     uint32_t dv_ind) {
   std::string boot_state;
-  std::tie(std::ignore, boot_state) = readTmpFile(dv_ind, "boot", "nps_mode");
+  std::tie(std::ignore, boot_state) = readTmpFile(dv_ind, "boot",
+                                                  "memory_partition");
   return boot_state;
 }
 
+
 #undef RET_IF_NONZERO
 }  // namespace smi
 }  // namespace amd
